import sys
import os
import gzip
import re
import hashlib
from optparse import OptionParser


class BennoFile:

    def __init__(self,filename):
        self.filename=filename
        self.cs_content=''
        self.cs_header=''
        self.mailcontent=''
        self.header=True

        f=gzip.open(filename,'rb')
        for line in f:
            m=re.search('^===== Hash:\s(\S+)',line)
            if m:
                self.cs_header=m.group(1)
                self.header=False
                break
        self.mailcontent=f.read()
        f.close()
        if self.header:
            raise ValueError('No Benno header found')
        else:
            sha=hashlib.sha256()
            sha.update(self.mailcontent)
            self.cs_content=sha.hexdigest().upper()

    def digestContent(self):
        return self.cs_content

    def digestMeta(self):
        return self.cs_header

    def getMail(self):
        return self.mailcontent

    def verifySelf(self):
        '''Verify content against header digest value'''
        if self.cs_header==self.cs_content:
            return True
        else:
            return False

    def verifyDigest(self,digest):
        '''Verify mail content against given digest'''
        ## remove padding tail '00'
        #if digest.endswith('00'):
        #    digest=digest[:-2]
        if self.cs_content==digest:
            return True
        else:
            return False

    def verifyFilenameHash(self,digest):
        '''Verify mail content against given digest'''
        # remove padding tail '00'
        if digest.endswith('00'):
            digest=digest[:-2]
        if self.cs_content==digest:
            return True
        else:
            return False


